from __future__ import annotations

from dataclasses import dataclass
from typing import Optional
import re

@dataclass(frozen=True)
class ModelPrice:
    input_per_million: float
    output_per_million: float


@dataclass(frozen=True)
class ModelKey:
    provider: str
    model: str

_DATE_SUFFIX = re.compile(r"-\d{4}-\d{2}-\d{2}$")

def canonicalize(provider: str, model: str) -> ModelKey:
    p = provider.strip().lower()

    m = model.strip()
    # ensure "openai/..." prefix is present if that's your convention
    if "/" not in m:
        m = f"{p}/{m}"

    # strip dated suffix: openai/gpt-4.1-2025-04-14 -> openai/gpt-4.1
    m = _DATE_SUFFIX.sub("", m)

    return ModelKey(provider=p, model=m)


class PricingRegistry:
    def __init__(self):
        self._prices: dict[ModelKey, ModelPrice] = {}

    def set_price(self, provider: str, model: str, *, input_per_million: float, output_per_million: float) -> None:
        key = canonicalize(provider, model)
        self._prices[key] = ModelPrice(
            input_per_million=float(input_per_million),
            output_per_million=float(output_per_million),
        )

    def get_price(self, provider: str, model: str) -> ModelPrice:
        key = canonicalize(provider, model)
        try:
            return self._prices[key]
        except KeyError as e:
            raise KeyError(f"No pricing configured for {key.provider}/{key.model}") from e


"""
Manual smoke-test runner for agent_meter.

Usage:
  OPENAI_API_KEY=... python -m agent_meter
"""

import asyncio
import os

from a2a_llm_tracker  import Meter, PricingRegistry, meter_context
from a2a_llm_tracker.integrations.litellm import LiteLLM
from a2a_llm_tracker.sinks.jsonl import JSONLSink
from dotenv import load_dotenv
load_dotenv()

def setup_meter() -> LiteLLM:
    pricing = PricingRegistry()

    # Example prices — adjust as needed
    pricing.set_price(
        provider="openai",
        model="openai/gpt-4.1",
        input_per_million=2.0,
        output_per_million=8.0,
    )

    meter = Meter(
        pricing=pricing,
        sinks=[JSONLSink("debug_usage.jsonl")],
        project="agent-meter-smoke-test",
    )

    return LiteLLM(meter=meter)


def run_sync(llm: LiteLLM) -> None:
    print("\n=== SYNC NON-STREAMING ===")
    resp = llm.completion(
        model="openai/gpt-4.1",
        messages=[{"role": "user", "content": "Say hello in one sentence."}],
    )
    print(resp)


def run_stream(llm: LiteLLM) -> None:
    print("\n=== SYNC STREAMING ===")
    for chunk in llm.completion(
        model="openai/gpt-4.1",
        messages=[{"role": "user", "content": "Write a short haiku about testing."}],
        stream=True,
    ):
        print(chunk, end="", flush=True)
    print()


async def run_async(llm: LiteLLM) -> None:
    print("\n=== ASYNC NON-STREAMING ===")
    resp = await llm.acompletion(
        model="openai/gpt-4.1",
        messages=[{"role": "user", "content": "Async hello!"}],
    )
    print(resp)


async def run_async_stream(llm: LiteLLM) -> None:
    print("\n=== ASYNC STREAMING ===")
    stream = await llm.acompletion(
        model="openai/gpt-4.1",
        messages=[{"role": "user", "content": "Stream something async."}],
        stream=True,
    )
    async for chunk in stream:
        print(chunk, end="", flush=True)
    print()


def main() -> None:
    if not os.getenv("OPENAI_API_KEY"):
        print("⚠️  OPENAI_API_KEY is not set. Exiting.")
        return

    llm = setup_meter()

    with meter_context(agent_id="smoke-test-agent", session_id="local"):
        run_sync(llm)
        run_stream(llm)
        asyncio.run(run_async(llm))
        asyncio.run(run_async_stream(llm))

    print("\n✔ Smoke test complete. Check debug_usage.jsonl")


if __name__ == "__main__":
    main()


from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import Any, Optional


def utcnow() -> datetime:
    return datetime.now(timezone.utc)


@dataclass
class UsageEvent:
    provider: str
    model: str

    input_tokens: Optional[int] = None
    output_tokens: Optional[int] = None
    total_tokens: Optional[int] = None

    input_cost_usd: Optional[float] = None
    output_cost_usd: Optional[float] = None
    cost_usd: Optional[float] = None

    accuracy: str = "unknown"  # "exact" | "estimated" | "unknown"

    request_id: Optional[str] = None
    latency_ms: Optional[int] = None
    status: str = "ok"  # "ok" | "error"
    error_type: Optional[str] = None
    error_message: Optional[str] = None

    ts: datetime = field(default_factory=utcnow)

    # attribution
    agent_id: Optional[str] = None
    user_id: Optional[str] = None
    session_id: Optional[str] = None
    trace_id: Optional[str] = None

    metadata: dict[str, Any] = field(default_factory=dict)
